/****************************************************************************
**
** Copyright (C) 2007 Ralf Habacker. All rights reserved.
**
** This file is part of the KDE installer for windows
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is proviwded AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef GPMDATABASE_H
#define GPMDATABASE_H

#include <QObject>
#include "package.h"

namespace GPM {

/**
 holds local package database
 the local package database is build by package version files
 located in <installation-root>/manifest directory
 package version file names are build up by <package-name>-<version>-<packagetype>.ver
*/

typedef QStringList DatabasePackageList; 
typedef QString DatabasePackage; 

class Database: public QObject
{
    Q_OBJECT

public:
    Database ( QObject *parent = 0 );
    virtual ~Database();

    ///--- generic database access methods ---
    
    /// open package database located at dirpath
    /// have to set the root path internal
    bool open( const QString &dirPath=QString() );

    /// close package database 
    void close();

    /// create package database located at dirpath
    /// have to set the root path internal
    bool create( const QString &dirPath=QString() );

    ///--- find database entries ---

    /// search database by file 
    DatabasePackageList findByFile(const QString &fileName);
    
    /// search database by group/category 
    DatabasePackageList findByGroup(const QString &groupName);
    
    /// search database by packages
    DatabasePackageList findByPackage(const QString &packageName);

    /// search database by requires
    DatabasePackageList findByRequires(const QString &requires);
    
    /// search database by provides 
    DatabasePackageList findByProvides(const QString &Provides);

    ///--- package install/remove/verify

    /// install package 
    bool installPackage( const QString &filePath );

    /// remove package 
    bool removePackage( const QString &packageName );

    /// remove package 
    bool verifyPackage( const QString &packageName );
    
    void listPackageFiles ( const QString &pkgName, Package::Type pkgType=Package::BIN );
    void listPackages ( const QString &title=QString() );
    QStringList getPackageFiles ( const QString &pkgName, Package::Type pkgType=Package::BIN );
    Package *getPackage ( const QString &pkgName, const QByteArray &version=QByteArray() );
    void clear();

    int size() const {
        return m_database.size();
    }

    const QList <Package*> &packages() const {
        return m_database;
    }

    void dump ( const QString &title=QString() );

    QString path() const {
        return m_root;
    }

    static bool isAnyPackageInstalled ( const QString &installRoot );

Q_SIGNALS:
    void configLoaded();

protected:
    void addFromRegistry();

protected Q_SLOTS:
    void slotInstallDirChanged ( const QString & );

private:
    QMap<QString,QString> m_installKeys;
    QMap<QString,QString> m_versionKeys;
    QList<Package*> m_database;
    QString m_root;

    friend QDebug & operator<< ( QDebug &, const Database & );
};

QDebug & operator<< ( QDebug &, const Database & );

}

#endif
